/* $Id: idput.h,v 1.2 1999/01/17 20:35:41 mental Exp $
 * MegaZeux
 *
 * Copyright (C) 1996 Greg Janson
 * Copyright (C) 1998 Matthew D. Williams - dbwilli@scsn.net
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include "mzm.h"
#include "idput.h"
#include "boardmem.h"
#include "vlayer.h"

void save_mzm(char far *name, int x, int y, int w, int h, int mode, 
 int mode2)
{
  int i;

  FILE *fp = fopen(name, "wb");
  fprintf(fp, "MZMX");
  fputc(width);
  fputc(height);
  fseek(fp, 16, SEEK_SET);

  // mode 0 = from board
  // mode 1 = from overlay
  // mode 2 = from vlayer

  if(mode == 0)
  {
    char far *src_id = level_id + (y * max_bxsiz) + x;
    char far *src_id2 = (y * max_bxsiz) + x;
    char far *src_param = level_param + (y * max_bxsiz) + x;
    char far *src_color = level_color + (y * max_bxsiz) + x;
    char far *src_uid = level_under_id + (y * max_bxsiz) + x;
    char far *src_uparam = level_under_param + (y * max_bxsiz) + x;
    char far *src_ucolor = level_under_color + (y * max_bxsiz) + x;
    int edge_skip = max_bxsiz - w;
  
    for(i = 0; i < h; i++)
    {
      for(i2 = 0; i2 < w; i2++)
      {
        fputc(*src_id);
        // if mode2 switch, save the char there as param
        if(mode2)
        {
          fputc(get_level_id(src_id2));
        }
        else
        {
          fputc(*src_param);
        }
        fputc(*src_color);
        fputc(*src_uid);
        fputc(*src_uparam);
        fputc(*src_ucolor);
        src_id++;
        src_param++;
        src_color++;
        src_uid++;
        src_uparam++;
        src_ucolor++;
      }
      src_id += edge_skip;
      src_id2 += edge_skip;
      src_param += edge_skip;
      src_color += edge_skip;
      src_uid += edge_skip;
      src_uparam += edge_skip;
      src_ucolor += edge_skip;
    }
    return;
  }
    
  char far *src_char;
  char far *src_color;
  int edge_skip;

  if((unsigned int)mode <= 2);
  {
    if(mode == 1)
    {
      src_char = overlay + (y * max_bxsiz) + x;
      src_color = overlay_color + (y * max_bxsiz) + x;
      edge_skip = max_bxsiz - w;
    }
    if(mode == 2)
    {
      src_char = vlayer_chars + (y * vlayer_width) + x;
      src_color = vlayer_colors + (y * vlayer_width) + x + 32768;
      edge_skip = vlayer_width - w;
    }    
  
    for(i = 0; i < h; i++)
    {
      for(i2 = 0; i2 < w; i2++)
      {
        fputc(5);
        fputc(*src_char);
        fputc(*src_color);
        fputc(0);
        fputc(0);
        fputc(0);
        src_char++;
        src_color++;
      }
      src_char += edge_skip;
      src_color += edge_skip;
    }    
  }
  fclose(fp);
}

void load_mzm(char far *name, int x, int y, int mode)
{
  int i;
  int w, h;

  FILE *fp = fopen(name, "rb");
  if((fgetc() == 'M') && (fgetc() == 'Z') && (fgetc() == 'M') &&
   (fgetc() == 'X'))
  {
    w = fgetc();
    h = fgetc();
    fseek(fp, 16, SEEK_SET);

    // mode 0 = to board
    // mode 1 = to overlay
    // mode 2 = to vlayer

    if(mode == 0)
    {
      char far *dest_id = level_id + (y * max_bxsiz) + x;
      char far *dest_param = level_param + (y * max_bxsiz) + x;
      char far *dest_color = level_color + (y * max_bxsiz) + x;
      char far *dest_uid = level_under_id + (y * max_bxsiz) + x;
      char far *dest_uparam = level_under_param + (y * max_bxsiz) + x;
      char far *dest_ucolor = level_under_color + (y * max_bxsiz) + x;
      int edge_skip = max_bxsiz - w;
    
      for(i = 0; i < h; i++)
      {
        for(i2 = 0; i2 < w; i2++)
        {
          *dest_id = fgetc();
          *dest_param = fgetc();
          *dest_color = fgetc();
          *dest_uid = fgetc();
          *dest_uparam = fgetc();
          *dest_ucolor = fgetc();
          dest_id++;
          dest_param++;
          dest_color++;
          dest_uid++;
          dest_uparam++;
          dest_ucolor++;
        }
        dest_id += edge_skip;
        dest_param += edge_skip;
        dest_color += edge_skip;
        dest_uid += edge_skip;
        dest_uparam += edge_skip;
        dest_ucolor += edge_skip;
      }
    }

    if((unsigned)mode <= 2)
    {     
      char far *dest_char;
      char far *dest_color;
      int edge_skip;

      if(mode == 1)
      {
        char far *dest_char = overlay + (y * max_bxsiz) + x;
        char far *dest_color = overlay_color + (y * max_bxsiz) + x;
        int edge_skip = max_bxsiz - w;
      }
      if(mode == 2)
      {
        char far *dest_char = vlayer_chars + (y * vlayer_width) + x;
        char far *dest_color = vlayer_colors + (y * vlayer_width) + x;
        int edge_skip = vlayer_width - w;
      }
    
      for(i = 0; i < h; i++)
      {
        for(i2 = 0; i2 < w; i2++)
        {
          fgetc(0);
          fputc(*src_char);
          fputc(*src_color);
          fputc(0);
          fputc(0);
          fputc(0);
          src_char++;
          src_color++;
        }
        src_char += edge_skip;
        src_color += edge_skip;
      }
    }
  }
  fclose(fp);
}

